--
-- FuelAdjustFix
-- Specialization to deactivate the 'FuelAdjust' Mod from Blacky for the Pony
--
-- @author  	Ifko[nator]
-- @date  		15.01.2018
-- @version		1.0


FuelAdjustFix = {};

local modFilename, _, _ = Utils.removeModDirectory(g_currentModDirectory);

function FuelAdjustFix:loadMap(name)
	self.modName = "";
	self.modNamePrint = string.sub(modFilename, 0, (string.len(modFilename) - 1));
	
	for _, mod in pairs(ModsUtil.modList) do
		if SpecializationUtil.getSpecialization(tostring(mod.modName) .. ".FuelAdjust") ~= nil then		
			print("[Info from the FuelAdjustFix.lua]: Found the FuelAdjust.lua in the '" .. mod.modName ..".zip'.");
			
			if g_modIsLoaded[tostring(mod.modName)] then	
				self.modName = _G[tostring(mod.modName)];
			end;
		end;
	end;
	
	if self.modName ~= "" then
		self.modName.FuelAdjust.load = Utils.overwrittenFunction(self.modName.FuelAdjust.load, FuelAdjustFix.loadFix);
		self.modName.FuelAdjust.poatLoad = Utils.overwrittenFunction(self.modName.FuelAdjust.postLoad, FuelAdjustFix.postLoadFix);
		self.modName.FuelAdjust.update = Utils.overwrittenFunction(self.modName.FuelAdjust.update, FuelAdjustFix.updateFix);
		self.modName.FuelAdjust.updateTick = Utils.overwrittenFunction(self.modName.FuelAdjust.updateTick, FuelAdjustFix.updateTickFix);
		self.modName.FuelAdjust.draw = Utils.overwrittenFunction(self.modName.FuelAdjust.draw, FuelAdjustFix.drawFix);
		self.modName.FuelAdjust.readStream = Utils.overwrittenFunction(self.modName.FuelAdjust.readStream, FuelAdjustFix.readStreamFix);
		self.modName.FuelAdjust.writeStream = Utils.overwrittenFunction(self.modName.FuelAdjust.writeStream, FuelAdjustFix.writeStreamFix);
		
		print("[Info from the FuelAdjustFix.lua]: Overwrite some functions from the FuelAdjust.lua to stop this for the '" .. self.modNamePrint ..".zip'.");
	else
		print("[Info from the FuelAdjustFix.lua]: NOT overwrite some functions from the FuelAdjust.lua to stop this for the '" .. self.modNamePrint ..".zip', because the fuel adjust mod was not loaded!");
	end;
end;

function FuelAdjustFix:loadFix(superFunc)
	if self.pony == nil then
		superFunc(self);
	else
		self.fa = {};
	end;
end;

function FuelAdjustFix:postLoadFix(superFunc)
	if self.pony == nil then
		superFunc(self);
	end;
end;

function FuelAdjustFix:updateFix(superFunc, dt)
	if self.pony == nil then
		superFunc(self, dt);
	end;
end;

function FuelAdjustFix:updateTickFix(superFunc, dt)
	if self.pony == nil then
		superFunc(self, dt);
	end;
end;

function FuelAdjustFix:drawFix(superFunc)
	if self.pony == nil then
		superFunc(self);
	end;
end;

function FuelAdjustFix:readStreamFix(superFunc, streamId, connection)
	if self.pony == nil then
		superFunc(self, streamId, connection);
	end;
end;

function FuelAdjustFix:writeStreamFix(superFunc, streamId, connection)
	if self.pony == nil then
		superFunc(self, streamId, connection);
	end;
end; 

--## Unused but needed functions

function FuelAdjustFix:update(dt)end;
function FuelAdjustFix:deleteMap()end;
function FuelAdjustFix:keyEvent(unicode, sym, modifier, isDown)end;
function FuelAdjustFix:mouseEvent(posX, posY, isDown, isUp, button)end;
function FuelAdjustFix:draw()end;

addModEventListener(FuelAdjustFix);