--[[
RideablePony

Author:	 Ifko[nator]
Date:	 19.01.2018
Version: 1.0 

History: V1.0 @ 19.01.2018 - initial release in FS 17
]]

RideablePony = {};

function RideablePony.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function RideablePony:load(savegame)
    self.pony = {};
	
	self.pony.index = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.rideablePony.animation#index"));
	self.pony.foodFillUnit = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.rideablePony#foodFillUnit"), 1);
	self.pony.waterFillUnit = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.rideablePony#waterFillUnit"), 2);
    
	self.pony.animation = {};
	
	self.pony.animation.node = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.rideablePony.animation#node"));
    
	self.pony.animation.walkClip = getXMLString(self.xmlFile, "vehicle.rideablePony.animation.clips.walk#name");
	self.pony.animation.runClip = getXMLString(self.xmlFile, "vehicle.rideablePony.animation.clips.run#name");
	self.pony.animation.idleClip = getXMLString(self.xmlFile, "vehicle.rideablePony.animation.clips.idle#name");
	self.pony.animation.feedingClip = getXMLString(self.xmlFile, "vehicle.rideablePony.animation.clips.feeding#name");
	
	self.pony.animation.speed = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.rideablePony.animation#speed"), 45);
	self.pony.animation.speedBackup = self.pony.animation.speed;
	
	self.pony.saddleUpNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.rideablePony.bridle.saddleUp#node"));
	self.pony.bridleNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.rideablePony.bridle#node"));
	
	self.pony.foodUsage = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rideablePony#foodUsage"), 25) / 1000;
	self.pony.waterUsage = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.rideablePony#waterUsage"), 50) / 1000;	
	self.pony.currentMinute = g_currentMission.environment.currentMinute;
	
	self.pony.feedingPonyTargets = {};
	
    IKUtil.loadIKChainTargets(self.xmlFile, "vehicle.rideablePony.feeding", self.components, self.pony.feedingPonyTargets);
	
	self.pony.hasSaved = false;
	
	if savegame ~= nil then
		self.pony.hasSaved = Utils.getNoNil(getXMLString(savegame.xmlFile, savegame.key .. "#hasSaved"), self.pony.hasSaved);
	end;
	
	if self.isClient then
		self.pony.sampleJumpOn = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.jumpOn", nil, self.baseDirectory);
        self.pony.sampleJumpOff = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.jumpOff", nil, self.baseDirectory);
		self.pony.sampleStop = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.stop", nil, self.baseDirectory);
        self.pony.sampleWalk = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.walk", nil, self.baseDirectory, self.components[1].node);
		self.pony.sampleRun = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.run", nil, self.baseDirectory, self.components[1].node);
		self.pony.sampleNeigh = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.neigh", nil, self.baseDirectory, self.components[1].node);
		self.pony.sampleEat = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.eat", nil, self.baseDirectory, self.components[1].node);
		self.pony.sampleDrink = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.rideablePony.sounds.drink", nil, self.baseDirectory, self.components[1].node);
		
		--## load default variables
		local maxRpmDelta = (self.motor:getMaxRpm() - self.motor:getMinRpm());
        self.maxRpsDelta = maxRpmDelta / 60;
		
		self.sampleMotorStart = self.pony.sampleJumpOn;
		self.sampleMotorStop = self.pony.sampleJumpOff;
		self.sampleMotorRun = self.pony.sampleWalk;
		self.sampleHonk = self.pony.sampleNeigh;
		
		self.pony.doStopSound = false;
		
		if self.sampleMotorRun.sample ~= nil then	
			self.motorSoundRunPitchMaxWalk = 2.0;
			self.motorSoundRunPitchScaleWalk = (self.motorSoundRunPitchMaxWalk - self.pony.sampleWalk.pitchOffset) / self.maxRpsDelta;
			self.motorSoundRunMinimalVolumeFactorWalk = 0.0;
			
			self.motorSoundRunPitchMaxRun = 2.0;
			self.motorSoundRunPitchScaleRun = (self.motorSoundRunPitchMaxRun - self.pony.sampleRun.pitchOffset) / self.maxRpsDelta;
			self.motorSoundRunMinimalVolumeFactorRun = 0.0;
			
			self.motorSoundRunPitchMax = self.motorSoundRunPitchMaxWalk;
			self.motorSoundRunPitchScale = self.motorSoundRunPitchScaleWalk;
			self.motorSoundRunMinimalVolumeFactor = self.motorSoundRunMinimalVolumeFactorWalk;
		end;
	end;
	
	self.stopMotorOnLeave = false;
	self.isSelectable = true;
	
	--## let us adjust some default functions
	Motorized.draw = Utils.overwrittenFunction(Motorized.draw, RideablePony.motorizedDraw);
	Motorized.update = Utils.overwrittenFunction(Motorized.update, RideablePony.motorizedUpdate);
	
	Steerable.draw = Utils.overwrittenFunction(Steerable.draw, RideablePony.steerableDraw);
	
	self.getInteractionHelp = Utils.overwrittenFunction(self.getInteractionHelp, RideablePony.getInteractionHelp);
	
	Honk.draw = Utils.overwrittenFunction(Honk.draw, RideablePony.honkDraw);
	
	Fillable.draw = Utils.overwrittenFunction(Fillable.draw, RideablePony.fillableDraw);
	FillActivatable.updateActivateText = Utils.overwrittenFunction(FillActivatable.updateActivateText, RideablePony.fillableUpdateActivateText);
end;

function RideablePony:postLoad(savegame)
	if not self.pony.hasSaved then
		for fillUnitIndex, fillUnit in pairs(self.fillUnits) do	
			for _, fillType in pairs({"DRYGRASS_WINDROW", "WATER"}) do
				if self:allowUnitFillType(fillUnitIndex, FillUtil["FILLTYPE_" .. fillType], false) then
					self:setUnitFillLevel(fillUnitIndex, fillUnit.capacity, FillUtil["FILLTYPE_" .. fillType], true, nil);
				end;
			end;
		end;
		
		self.pony.hasSaved = true;
	end;
	
	if self.vehicleCharacter ~= nil then
		self.pony.ikChainTargetBackup = self.vehicleCharacter.ikChainTargets;
    end;
end;

function RideablePony:getSaveAttributesAndNodes(nodeIdent)
	return 'hasSaved="true"';
end;

function RideablePony:delete()
	if self.isClient then
		SoundUtil.deleteSample(self.pony.sampleJumpOn);
		SoundUtil.deleteSample(self.pony.sampleJumpOff);
		SoundUtil.deleteSample(self.pony.sampleStop);
		SoundUtil.deleteSample(self.pony.sampleWalk);
		SoundUtil.deleteSample(self.pony.sampleRun);
		SoundUtil.deleteSample(self.pony.sampleNeigh);
		SoundUtil.deleteSample(self.pony.sampleEat);
		SoundUtil.deleteSample(self.pony.sampleDrink);
	end;
end;


function RideablePony:update(dt)
	if self.pony ~= nil then
		if self.pony.index ~= nil 
			and self.pony.animation.node ~= nil 
			and self.pony.animation.idleClip ~= nil 
		then
			local ponyAnimationNode = getAnimCharacterSet(self.pony.animation.node);
			local ponyAnimationClip = getAnimClipIndex(ponyAnimationNode, self.pony.animation.idleClip);
			
			local movingDirection, lastMovedDistance, addedFoodConsumption, addedWaterConsumption = 1, 0.025, 1, 1;
			
			local hasEnoughFood = self:getUnitFillType(self.pony.foodFillUnit) == FillUtil.FILLTYPE_DRYGRASS_WINDROW; 
			local hasEnoughWater = self:getUnitFillType(self.pony.waterFillUnit) == FillUtil.FILLTYPE_WATER;
			local isPalletWithWater = self.fillTrigger ~= nil and self.fillTrigger.fillType == FillUtil.FILLTYPE_WATER;
			
			local getIsActive = self.isEntered 
				or self.isControlled 
				or self:getIsHired() 
				or self.followMeIsStarted 
				or (self.getIsCourseplayDriving ~= nil and self:getIsCourseplayDriving())
			;		
		
			if not self:getIsMotorStarted() then
				--## start the 'motor' from the pony immediately after the purchase
				
				self:startMotor(true);
			end;
	
			local newX2, newY2, newZ2 = getWorldTranslation(self.pony.index);
			
			if self.pony.saddleUpNode ~= nil then
				setVisibility(self.pony.saddleUpNode, getIsActive and not (self.isFilling or self.isWaterTrailerFilling));
				
				if self.pony.bridleNode ~= nil then
					setVisibility(self.pony.bridleNode, getIsActive);
				end;
			end;
			
			if not self.isEntered then
				SoundUtil.stopSample(self.pony.sampleJumpOn);
				
				SoundUtil.stop3DSample(self.pony.sampleJumpOn);
			end;
			
			if getIsActive then
				if self.lastTranslationR == nil then
					self.lastTranslationR = {newX2, newY2, newZ2};
				end;
				
				local dx2, dy2, dz2 = worldDirectionToLocal(self.pony.index, newX2 - self.lastTranslationR[1], newY2 - self.lastTranslationR[2], newZ2 - self.lastTranslationR[3]);
				local runSpeed = 0;
				local runSound = self.pony.sampleWalk;
				
				movingDirection = 0;
				lastMovedDistance = Utils.vector3Length(dx2, dy2, dz2);
				
				if self.lastSpeedReal > 1.0E-5 then    
					if dz2 < 1.0E-4 then
						movingDirection = -1;
					elseif dz2 > 1.0E-4 then
						movingDirection = 1;        
					end;
				end;
				
				if self.isClient then
					if self.pony.animation.feedingClip ~= nil then
						if self.isFilling and self.pony.sampleEat ~= nil and not isPalletWithWater then
							SoundUtil.play3DSample(self.pony.sampleEat);
						else
							SoundUtil.stop3DSample(self.pony.sampleEat);
						end;
						
						if self.isFilling and self.pony.sampleDrink ~= nil and isPalletWithWater then
							SoundUtil.play3DSample(self.pony.sampleDrink);
						else
							SoundUtil.stop3DSample(self.pony.sampleDrink);
						end;
					end;
					
					if self.vehicleCharacter ~= nil then
						self.vehicleCharacter.ikChainTargets = self.pony.ikChainTargetBackup;
						
						if self.isFilling and self.pony.animation.feedingClip ~= nil then
							self.vehicleCharacter.ikChainTargets = self.pony.feedingPonyTargets;
						end;
						
						if self.vehicleCharacter.isCharacterLoaded then
							for ikChainId, target in pairs(self.vehicleCharacter.ikChainTargets) do
								IKUtil.setTarget(self.vehicleCharacter.ikChains, ikChainId, target);
							end;
						end;
						
						self.vehicleCharacter:setDirty();
					end;
				end;
				
				if self.isFilling and self.pony.animation.feedingClip ~= nil then
					--## feeding animation
					
					ponyAnimationClip = getAnimClipIndex(ponyAnimationNode, self.pony.animation.feedingClip);
					
					lastMovedDistance, movingDirection = 0.025, 1;
					
					if self.isServer then
						Drivable.updateVehiclePhysics(self, 0, self.axisForwardIsAnalog, 0, self.axisSideIsAnalog, true, dt);
					end;
				elseif self.lastSpeedReal < 1.0E-5 then
					--## idle animation (already set in line 161)
					
					lastMovedDistance, movingDirection = 0.025, 1;
				elseif self:getLastSpeed() > 12.5 and self.pony.animation.runClip ~= nil then
					--## run animation
					
					ponyAnimationClip = getAnimClipIndex(ponyAnimationNode, self.pony.animation.runClip);
					
					runSpeed = 145/self:getLastSpeed();
					
					addedFoodConsumption, addedWaterConsumption = 1.8, 2.1;
				elseif self.pony.animation.walkClip ~= nil then
					--## walk animation
					
					ponyAnimationClip = getAnimClipIndex(ponyAnimationNode, self.pony.animation.walkClip);
					
					addedFoodConsumption, addedWaterConsumption = 1.4, 1.7;
				end;    
				
				if runSpeed > 0 then
					self.pony.animation.speed = runSpeed;
					runSound = self.pony.sampleRun;
					
					self.motorSoundRunPitchMax = self.motorSoundRunPitchMaxRun;
					self.motorSoundRunPitchScale = self.motorSoundRunPitchScaleRun;
					self.motorSoundRunMinimalVolumeFactor = self.motorSoundRunMinimalVolumeFactorRun;
					
					if not self.pony.doStopSound then
						SoundUtil.stopSample(self.sampleMotorRun);
						SoundUtil.stop3DSample(self.sampleMotorRun);
						
						self.pony.doStopSound = true;
					end;
				else
					self.pony.animation.speed = self.pony.animation.speedBackup;
					
					if movingDirection == 1 then
						self.motorSoundRunPitchMax = self.motorSoundRunPitchMaxWalk;
						self.motorSoundRunPitchScale = self.motorSoundRunPitchScaleWalk;
					else
						self.motorSoundRunPitchMax = self.motorSoundRunPitchMaxWalk / 2.5;
						self.motorSoundRunPitchScale = self.motorSoundRunPitchScaleWalk / 2.5;
					end;
						
					self.motorSoundRunMinimalVolumeFactor = self.motorSoundRunMinimalVolumeFactorWalk;
					
					if self.pony.doStopSound then
						SoundUtil.stopSample(self.sampleMotorRun);
						SoundUtil.stop3DSample(self.sampleMotorRun);
						
						self.pony.doStopSound = false;
					end;
				end;
				
				self.sampleMotorRun = runSound;
				
				self.lastTranslationR = {newX2, newY2, newZ2};
				
				local maxSpeed = 0;
			
				if hasEnoughWater and not hasEnoughFood then
					maxSpeed = 20;
				elseif not hasEnoughWater and hasEnoughFood then
					maxSpeed = 15;
				elseif not hasEnoughWater and not hasEnoughFood then
					maxSpeed = 10;
				end;
				
				if maxSpeed ~= 0 then
					--## set max speed down if water, food or both is empty
					
					if self.cruiseControl ~= nil and self.cruiseControl.state == Drivable.CRUISECONTROL_STATE_ACTIVE then
						maxSpeed = math.min(maxSpeed, self.cruiseControl.speed);
					end;
					
					self.motor:setSpeedLimit(maxSpeed);
				end;
			else
				SoundUtil.stopSample(self.sampleMotorRun);
				SoundUtil.stop3DSample(self.sampleMotorRun);
				
				if self.isFilling then
					if isPalletWithWater then	
						SoundUtil.stop3DSample(self.pony.sampleDrink);
					else	
						SoundUtil.stop3DSample(self.pony.sampleEat);
					end;
					
					self:setIsFilling(false);
				end;
			end;
			
			assignAnimTrackClip(ponyAnimationNode , 0, ponyAnimationClip);
			setAnimTrackLoopState(ponyAnimationNode, 0, true);
			setAnimTrackSpeedScale(ponyAnimationNode, 0, self.pony.animation.speed * lastMovedDistance * movingDirection);
			enableAnimTrack(ponyAnimationNode, 0);
			
			if not self.isFilling and self.pony.currentMinute ~= g_currentMission.environment.currentMinute and (hasEnoughWater or hasEnoughFood) then
				--## food and water usage
				
				local foodUsagePerMinute = ((self.pony.foodUsage * addedFoodConsumption) * 2000 / (24 * 60)) / 2.8;
				local waterUsagePerMinute = ((self.pony.waterUsage * addedWaterConsumption) * 2000 / (24 * 60)) / 2.8;
				
				for _, fillUnit in pairs(self.fillUnits) do
					if fillUnit.currentFillType ~= FillUtil.FILLTYPE_UNKNOWN then
						if hasEnoughFood then
							self:setUnitFillLevel(self.pony.foodFillUnit, fillUnit.fillLevel - foodUsagePerMinute, fillUnit.currentFillType, false, nil);
						end;
						
						if hasEnoughWater then
							self:setUnitFillLevel(self.pony.waterFillUnit, fillUnit.fillLevel - waterUsagePerMinute, fillUnit.currentFillType, false, nil);
						end;
					end;
				end;
				
				self.pony.currentMinute = g_currentMission.environment.currentMinute;
			end;
			
			if self.isEntered and g_currentMission.surfaceSounds ~= nil then
				for _,surfaceSound in pairs(g_currentMission.surfaceSounds) do
					if surfaceSound.sample.sample ~= nil then
						if SoundUtil.isSamplePlaying(surfaceSound.sample) then
							--## stop surface sound from the pony
							
							SoundUtil.stopSample(surfaceSound.sample);
						end;
					end;
				end;
			end;
		end;
	end;
end;

function RideablePony:onEnter()
	if self:getIsActiveForSound() then
        SoundUtil.playSample(self.pony.sampleJumpOn, 1, 0, nil);
	else
		SoundUtil.play3DSample(self.pony.sampleJumpOn);
    end;
end;

function RideablePony:onLeave()
	if self:getIsActiveForSound() then
        SoundUtil.playSample(self.pony.sampleJumpOff, 1, 0, nil);
		SoundUtil.playSample(self.pony.sampleStop, 1, 0, nil);
	else
		SoundUtil.play3DSample(self.pony.sampleJumpOff);
		SoundUtil.play3DSample(self.pony.sampleStop);
    end;
end;

function RideablePony:honkDraw(superFunc)
    if self.isClient and self:getIsActiveForInput(true) then
		local text = "honk";
		
		if self.pony ~= nil then	
			text = "neigh";
		end;
		
		g_currentMission:addHelpButtonText(g_i18n:getText("action_" .. text), InputBinding.HONK, nil, GS_PRIO_VERY_LOW);
    end;
end;

function RideablePony:getInteractionHelp(superFunc)
    if self.interactionFlag == Vehicle.INTERACTION_FLAG_STEERABLE then
		local text = "enter";
		
		if self.pony ~= nil	then
			text = "jumpOn";
		end;
		
		return g_i18n:getText("action_" .. text);
    else
		if superFunc ~= nil then
            return superFunc(self);
        else
            return "";
        end;
    end;
end;

function RideablePony:steerableDraw(superFunc)
	if not self.isEntered then
        return;
    end
	
	if self.pony == nil then
		if superFunc ~= nil then
			superFunc(self);
		end;
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("action_jumpOff"), InputBinding.ENTER, nil, GS_PRIO_NORMAL);
	end;
end;

function RideablePony:fillableDraw(superFunc)
	if self.pony == nil and superFunc ~= nil then
		superFunc(self);
	end;
end;

function RideablePony:fillableUpdateActivateText(superFunc)
	if self.fillable.pony == nil then
		superFunc(self);
	else
		if self.fillable.isFilling then
			self.activateText = g_i18n:getText("action_stopFeeding");
		else
			self.activateText = g_i18n:getText("action_startFeeding");
		end;
	end;
end;

function RideablePony:motorizedDraw(superFunc)
	if self.pony == nil and superFunc ~= nil then
		superFunc(self);
	end;
end;

function RideablePony:motorizedUpdate(superFunc, dt)
	--## lock the motor state to started for the pony
	
	if self.isClient 
        and self.isEntered 
		and self:getIsActiveForInput(false) 
		and not g_currentMission.missionInfo.automaticMotorStartEnabled 
		and self.pony == nil
	then
        if InputBinding.hasEvent(InputBinding.TOGGLE_MOTOR_STATE) then
            if not self:getIsHired() then
                if self.isMotorStarted then
                    self:stopMotor();
                else
                    self:startMotor();
                end;
            end;
        end;
    end;

    Utils.updateRotationNodes(self, self.motorTurnedOnRotationNodes, dt, self:getIsMotorStarted());

    if self:getIsMotorStarted() then
        local accInput = 0;
		
        if self.axisForward ~= nil then
            accInput = -self.axisForward;
        end;
		
        if self.cruiseControl ~= nil and self.cruiseControl.state ~= Drivable.CRUISECONTROL_STATE_OFF then
            accInput = 1;
        end;
		
        if self.isClient then
            if self:getIsActiveForSound() then
                if not SoundUtil.isSamplePlaying(self.sampleMotorStart, 1.5 * dt) then
                    SoundUtil.playSample(self.sampleMotor, 0, 0, nil);
					SoundUtil.playSample(self.sampleMotorRun, 0, 0, 0);
                    SoundUtil.playSample(self.sampleMotorLoad, 0, 0, 0);
                    SoundUtil.playSample(self.sampleGearbox, 0, 0, 0);
                    SoundUtil.playSample(self.sampleRetarder, 0, 0, 0);

                    if self.brakeLightsVisibility then
                        self.brakeLightsVisibilityWasActive = true;
                        self.maxDecelerationDuringBrake = math.max(self.maxDecelerationDuringBrake, math.abs(accInput));
                    end;
					
                    if self.brakeLightsVisibilityWasActive and not self.brakeLightsVisibility then
                        self.brakeLightsVisibilityWasActive = false;

                        local factor = self.maxDecelerationDuringBrake;
                        self.maxDecelerationDuringBrake = 0;

                        local airConsumption = self:getMaximalAirConsumptionPerFullStop();
						
                        airConsumption = factor * airConsumption;
                        self.brakeCompressor.fillLevel = math.max(0, self.brakeCompressor.fillLevel - airConsumption);
                    end;

                    if self.brakeCompressor.fillLevel < self.brakeCompressor.refillFilllevel then
                        self.brakeCompressor.doFill = true;
                    end;
					
                    if self.brakeCompressor.doFill and self.brakeCompressor.fillLevel == self.brakeCompressor.capacity then
                        self.brakeCompressor.doFill = false;
                    end;
					
                    if self.brakeCompressor.doFill then
                        self.brakeCompressor.fillLevel = math.min(self.brakeCompressor.capacity, self.brakeCompressor.fillLevel + self.brakeCompressor.fillSpeed * dt);
                    end;

                    if Vehicle.debugRendering then
                        renderText(0.3, 0.16, getCorrectTextSize(0.02), string.format("brakeCompressor.fillLevel = %.1f", 100*(self.brakeCompressor.fillLevel / self.brakeCompressor.capacity)));
                    end;

                    if not self.brakeCompressor.doFill then
                        if self.brakeCompressor.runSoundActive then
                            SoundUtil.stopSample(self.sampleBrakeCompressorRun, true);
                            SoundUtil.playSample(self.sampleBrakeCompressorStop, 1, 0, nil);
                            self.brakeCompressor.startSoundPlayed = false;
                            self.brakeCompressor.runSoundActive = false;
                        end;
                    elseif not SoundUtil.isSamplePlaying(self.sampleBrakeCompressorStop, 1.5*dt) then
                        if not self.brakeCompressor.startSoundPlayed then
                            self.brakeCompressor.startSoundPlayed = true;
                            SoundUtil.playSample(self.sampleBrakeCompressorStart, 1, 0, nil);
                        else
                            if not SoundUtil.isSamplePlaying(self.sampleBrakeCompressorStart, 1.5*dt) and not self.brakeCompressor.runSoundActive then
                                self.brakeCompressor.runSoundActive = true;
                                SoundUtil.playSample(self.sampleBrakeCompressorRun, 0, 0, nil);
                            end;
                        end;
                    end;
                end;

                if self.compressionSoundTime <= g_currentMission.time then
                    SoundUtil.playSample(self.sampleAirReleaseValve, 1, 0, nil);
                    
					self.compressionSoundTime = g_currentMission.time + math.random(10000, 40000);
                end;

                if self.sampleCompressedAir.sample ~= nil then
                    if self.movingDirection > 0 and self.lastSpeed > self.motor:getMaximumForwardSpeed() * 0.0002 then --## faster than 20% of max speed
                        if accInput < -0.05 then
                            --## play the compressor sound if we drive fast enough and brake
                            if not self.compressedAirSoundEnabled then
                                SoundUtil.playSample(self.sampleCompressedAir, 1, 0, nil);
                                
								self.compressedAirSoundEnabled = true;
                            end
                        else
                            self.compressedAirSoundEnabled = false;
                        end;
                    end;
                end;

                SoundUtil.stop3DSample(self.sampleMotor);
				SoundUtil.stop3DSample(self.sampleMotorRun);
                SoundUtil.stop3DSample(self.sampleGearbox);
                SoundUtil.stop3DSample(self.sampleRetarder);
            else
                SoundUtil.play3DSample(self.sampleMotor);
				SoundUtil.play3DSample(self.sampleMotorRun);
            end;

            --## adjust pitch and volume of samples
            if (self.wheels ~= nil and #self.wheels > 0) or (self.dummyWheels ~= nil and #self.dummyWheels > 0) then

                if self.sampleReverseDrive.sample ~= nil then
                    if (accInput < 0 or accInput == 0) and (self:getLastSpeed() > 3 and self.movingDirection ~= self.reverserDirection) then
                        if self:getIsActiveForSound() then
                            SoundUtil.playSample(self.sampleReverseDrive, 0, 0, nil);
                        end;
                    else
                        SoundUtil.stopSample(self.sampleReverseDrive);
                    end;
                end;

                local minRpm = self.motor:getMinRpm();
                local maxRpm = self.motor:getMaxRpm();

                local maxSpeed;
				
                if self.movingDirection >= 0 then
                    maxSpeed = self.motor:getMaximumForwardSpeed() * 0.001;
                else
                    maxSpeed = self.motor:getMaximumBackwardSpeed() * 0.001;
                end;

                local motorRpm = self.motor:getEqualizedMotorRpm();
				
                --## increase the motor rpm to the max rpm if faster than 75% of the full speed
                if self.movingDirection > 0 and self.lastSpeed > 0.75 * maxSpeed and motorRpm < maxRpm then
                    motorRpm = motorRpm + (maxRpm - motorRpm) * math.min((self.lastSpeed - 0.75 * maxSpeed) / (0.25 * maxSpeed), 1);
                end;
				
                --## the actual rpm offset is 50% from the motor and 50% from the speed
                local targetRpmOffset = (motorRpm - minRpm)*0.5 + math.min(self.lastSpeed / maxSpeed, 1) * (maxRpm - minRpm) *0.5;

                if Vehicle.debugRendering then
                    renderText(0.3, 0.14, getCorrectTextSize(0.02), string.format("getLastMotorRpm() = %.2f", self.motor:getLastMotorRpm()));
                    renderText(0.3, 0.12, getCorrectTextSize(0.02), string.format("getEqualziedMotorRpm() = %.2f", self.motor:getEqualizedMotorRpm()));
                    renderText(0.3, 0.10, getCorrectTextSize(0.02), string.format("targetRpmOffset = %.2f", targetRpmOffset));
                end;

                local alpha = math.pow(0.01, dt*0.001);
                local roundPerMinute = targetRpmOffset + alpha * (self.lastRoundPerMinute-targetRpmOffset);

                self.lastRoundPerMinute = roundPerMinute;

                local roundPerSecondSmoothed = roundPerMinute / 60;

                if self.sampleMotor.sample ~= nil then
                    local motorSoundPitch = math.min(self.sampleMotor.pitchOffset + self.motorSoundPitchScale * math.abs(roundPerSecondSmoothed), self.motorSoundPitchMax);
                    
					SoundUtil.setSamplePitch(self.sampleMotor, motorSoundPitch);
					
                    local deltaVolume = (self.sampleMotor.volume - self.motorSoundVolumeMin) * math.max(0.0, math.min(1.0, self:getLastSpeed() / self.motorSoundVolumeMinSpeed));
                    
					SoundUtil.setSampleVolume(self.sampleMotor, math.max(self.motorSoundVolumeMin, self.sampleMotor.volume - deltaVolume));
                end;

                if self.sampleMotorRun.sample ~= nil then
                    local motorSoundRunPitch = math.min(self.sampleMotorRun.pitchOffset + self.motorSoundRunPitchScale * math.abs(roundPerSecondSmoothed), self.motorSoundRunPitchMax);
                    
					SoundUtil.setSamplePitch(self.sampleMotorRun, motorSoundRunPitch);
					
                    local runVolume = roundPerMinute / (maxRpm - minRpm);
                    
					if math.abs(accInput) < 0.01 or Utils.sign(accInput) ~= self.movingDirection or ptoVolume == 0 then
                        runVolume = runVolume * 0.9;
                    end;
					
                    runVolume = Utils.clamp(runVolume, 0.0, 1.0);

                    if Vehicle.debugRendering then
                        renderText(0.3, 0.08, getCorrectTextSize(0.02), string.format("runVolume = %.2f", runVolume));
                    end;

                    if self.sampleMotorLoad.sample == nil then
                        SoundUtil.setSampleVolume(self.sampleMotorRun, runVolume * self.sampleMotorRun.volume);
                    else
                        local motorSoundLoadPitch = math.min(self.sampleMotorLoad.pitchOffset + self.motorSoundLoadPitchScale * math.abs(roundPerSecondSmoothed), self.motorSoundLoadPitchMax);
                        
						if self.pony == nil then
							SoundUtil.setSamplePitch(self.sampleMotorLoad, motorSoundLoadPitch);
						end;

                        if self.motorSoundLoadFactor < self.actualLoadPercentage then
                            self.motorSoundLoadFactor = math.min(self.actualLoadPercentage, self.motorSoundLoadFactor + dt / 500);
                        elseif self.motorSoundLoadFactor > self.actualLoadPercentage then
                            self.motorSoundLoadFactor = math.max(self.actualLoadPercentage, self.motorSoundLoadFactor - dt / 750);
                        end;
						
                        if Vehicle.debugRendering then
                            renderText(0.3, 0.06, getCorrectTextSize(0.02), string.format("motorSoundLoadFactor = %.2f", self.motorSoundLoadFactor));
                        end;

                        SoundUtil.setSampleVolume(self.sampleMotorRun, math.max(self.motorSoundRunMinimalVolumeFactor, (1.0 - self.motorSoundLoadFactor) * runVolume * self.sampleMotorRun.volume));
                        SoundUtil.setSampleVolume(self.sampleMotorLoad, math.max(self.motorSoundLoadMinimalVolumeFactor, self.motorSoundLoadFactor * runVolume * self.sampleMotorLoad.volume));
                    end;
                end;
				
                local pitchInfluence = 0;
                local volumeInfluence = 0;
				
                for _,wheel in pairs(self.wheels) do
                    --## as in debug rendering, is this still correct?
                    local susp = (wheel.netInfo.y - wheel.netInfo.yMin) / wheel.suspTravel - 0.2; --## at yMin, we have -20% compression

                    if wheel.netInfo.lastSusp == nil then
                        wheel.netInfo.lastSusp = susp;
                    end;
					
                    local delta = susp - wheel.netInfo.lastSusp;

                    pitchInfluence = pitchInfluence + delta;
                    volumeInfluence = volumeInfluence + math.abs(delta);
                end;
				
                pitchInfluence = pitchInfluence / #self.wheels;
                volumeInfluence = volumeInfluence / #self.wheels;

                if pitchInfluence > self.pitchInfluenceFromWheels then
                    self.pitchInfluenceFromWheels = math.min(pitchInfluence, self.pitchInfluenceFromWheels + dt / 300);
                elseif pitchInfluence < self.pitchInfluenceFromWheels then
                    self.pitchInfluenceFromWheels = math.max(pitchInfluence, self.pitchInfluenceFromWheels - dt / 300);
                end;

                if volumeInfluence > self.volumeInfluenceFromWheels then
                    self.volumeInfluenceFromWheels = math.min(volumeInfluence, self.volumeInfluenceFromWheels + dt / 300);
                elseif volumeInfluence < self.volumeInfluenceFromWheels then
                    self.volumeInfluenceFromWheels = math.max(volumeInfluence, self.volumeInfluenceFromWheels - dt / 300);
                end;

                if self.sampleGearbox.sample ~= nil then
                    local speedFactor = Utils.clamp( (self:getLastSpeed() - 1) / math.ceil(self.motor:getMaximumForwardSpeed() * 3.6), 0, 1);
                    local pitchGearbox = Utils.lerp(self.sampleGearbox.pitchOffset, self.gearboxSoundPitchMax, speedFactor ^ self.gearboxSoundPitchExponent);
                    local volumeGearbox = Utils.lerp(self.sampleGearbox.volume, self.gearboxSoundVolumeMax, speedFactor);

                    if self.reverserDirection ~= self.movingDirection then
                        speedFactor = Utils.clamp( (self:getLastSpeed() - 1) / math.ceil(self.motor:getMaximumBackwardSpeed() * 3.6), 0, 1);
                        pitchGearbox = Utils.lerp(self.sampleGearbox.pitchOffset, self.gearboxSoundReversePitchMax, speedFactor ^ self.gearboxSoundPitchExponent);
                        volumeGearbox = Utils.lerp(self.sampleGearbox.volume, self.gearboxSoundReverseVolumeMax, speedFactor);
                    end;
					
					SoundUtil.setSamplePitch(self.sampleGearbox, pitchGearbox);
                    SoundUtil.setSampleVolume(self.sampleGearbox, volumeGearbox);
                end;

                if self.sampleRetarder.sample ~= nil then
                    local speedFactor = Utils.clamp( (self:getLastSpeed() - self.retarderSoundMinSpeed) / math.ceil(self.motor:getMaximumForwardSpeed() * 3.6), 0, 1);
                    local pitchGearbox = Utils.lerp(self.sampleRetarder.pitchOffset, self.retarderSoundPitchMax, speedFactor);
                    
					if self.pony == nil then
						SoundUtil.setSamplePitch(self.sampleRetarder, pitchGearbox);
					end;

                    local volumeRetarder = Utils.lerp(self.sampleRetarder.volume, self.retarderSoundVolumeMax, speedFactor);
                    local targetVolume = 0.0;
					
                    if accInput <= 0.0 and self:getLastSpeed() > self.retarderSoundMinSpeed and self.reverserDirection == self.movingDirection then
                        if accInput > -0.9 then
                            targetVolume = volumeRetarder;
                        else
                            targetVolume = self.sampleRetarder.volume;
                        end;
                    end;

                    if self.retarderSoundActualVolume < targetVolume then
                        self.retarderSoundActualVolume = math.min(targetVolume, self.retarderSoundActualVolume + dt / self.axisSmoothTime);
                    elseif self.retarderSoundActualVolume > targetVolume then
                        self.retarderSoundActualVolume = math.max(targetVolume, self.retarderSoundActualVolume - dt / self.axisSmoothTime);
                    end;
					
                    SoundUtil.setSampleVolume(self.sampleRetarder, self.retarderSoundActualVolume);

                    if Vehicle.debugRendering then
                        renderText(0.8, 0.44, getCorrectTextSize(0.02), string.format("retarderSoundActualVolume = %.2f", self.retarderSoundActualVolume));
                        renderText(0.8, 0.42, getCorrectTextSize(0.02), string.format("getLastSpeed() = %.2f", self:getLastSpeed()));
                    end;
                end;

                if self.sampleBrakeCompressorRun.sample ~= nil then
                    local pitchCompressor = math.min(self.sampleBrakeCompressorRun.pitchOffset + self.brakeCompressorRunSoundPitchScale * math.abs(roundPerSecondSmoothed), self.brakeCompressorRunSoundPitchMax);
                    
					SoundUtil.setSamplePitch(self.sampleBrakeCompressorRun, pitchCompressor);
                end;
            end;
        end;

        if self.isServer then
            if not self:getIsHired() then
                if self.lastMovedDistance > 0 then
                    g_currentMission.missionStats:updateStats("traveledDistance", self.lastMovedDistance * 0.001);
                end;
            end;

            self:updateFuelUsage(dt);
        end;
    end;
end;

--## unused but needed functions

function RideablePony:updateTick(dt)end;
function RideablePony:mouseEvent(posX, posY, isDown, isUp, button)end;
function RideablePony:keyEvent(unicode, sym, modifier, isDown)end;
function RideablePony:draw()end;